import { ImageLike, Page } from 'tesseract.js';
export declare enum LoadingStatus {
    CORE_LOADING = "loading tesseract core",
    OCR_INITIALIZATION = "initializing tesseract",
    LANGUAGE_LOADING = "loading language traineddata",
    API_INITIALIZATION = "initializing api",
    RECOGNIZING = "recognizing text"
}
declare type ProgressListener = (event: {
    status: LoadingStatus;
    progress: number;
}) => void;
declare type Options = {
    language?: string;
    lineClassName?: string;
    wordClassName?: string;
};
export default class Ocr {
    data: Page;
    language: string;
    lineClassName: string;
    wordClassName: string;
    progressListeners: ProgressListener[];
    minFontSize: number;
    /**
     * In the constructor you can define some default options.
     *
     * @param {Options} options Options for general configuration.
     */
    constructor({ language, lineClassName, wordClassName }?: Options);
    /**
     * Add event listeners for highlighting selected blocks and format selected text.
     */
    addSelectionListener(): void;
    /**
     * Removing event listeners for highlighting selected blocks and format selected text.
     */
    removeSelectionListener(): void;
    /**
     * Adding progress listener for ocr.
     *
     * @param {ProgressListener} listener Callback function.
     */
    addProgressListener(listener: ProgressListener): void;
    /**
     * Removing progress listener for ocr.
     *
     * @param {ProgressListener} listener Callback function that was added before.
     */
    removeProgressListener(listener: ProgressListener): void;
    /**
     * Parse image with ocr.
     *
     * @param {ImageLike} image The image instance.
     */
    parseImage(image: ImageLike): Promise<void>;
    private updateTextSize;
    private updateElementSize;
    private mapWords;
    private mapLine;
    private mapParagraph;
    /**
     * Append generated html to the container.
     *
     * @param {HTMLElement} container The container where elements should append.
     * @param {number} scale Scale of the container compared to original image size.
     */
    appendHtml(container: HTMLElement, scale?: number): void;
    private handleSelection;
    private handleMouseDown;
    private handleCopy;
}
export {};
