var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { createWorker } from 'tesseract.js';
import throttle from 'lodash.throttle';
export var LoadingStatus;
(function (LoadingStatus) {
    LoadingStatus["CORE_LOADING"] = "loading tesseract core";
    LoadingStatus["OCR_INITIALIZATION"] = "initializing tesseract";
    LoadingStatus["LANGUAGE_LOADING"] = "loading language traineddata";
    LoadingStatus["API_INITIALIZATION"] = "initializing api";
    LoadingStatus["RECOGNIZING"] = "recognizing text";
})(LoadingStatus || (LoadingStatus = {}));
var THROTTLE_DELAY = 100;
var WORD_CLASS_NAME = 'text-recognition-word';
var Ocr = /** @class */ (function () {
    /**
     * In the constructor you can define some default options.
     *
     * @param {Options} options Options for general configuration.
     */
    function Ocr(_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.language, language = _c === void 0 ? 'eng' : _c, _d = _b.lineClassName, lineClassName = _d === void 0 ? '' : _d, _e = _b.wordClassName, wordClassName = _e === void 0 ? '' : _e;
        this.data = null;
        this.language = null;
        this.lineClassName = '';
        this.wordClassName = '';
        this.progressListeners = [];
        this.minFontSize = 2;
        this.language = language;
        this.lineClassName = lineClassName;
        this.wordClassName = wordClassName;
    }
    /**
     * Add event listeners for highlighting selected blocks and format selected text.
     */
    Ocr.prototype.addSelectionListener = function () {
        document.addEventListener('selectionchange', throttle(this.handleSelection, THROTTLE_DELAY));
        document.addEventListener('mousedown', this.handleMouseDown);
        document.addEventListener('copy', this.handleCopy);
    };
    /**
     * Removing event listeners for highlighting selected blocks and format selected text.
     */
    Ocr.prototype.removeSelectionListener = function () {
        document.removeEventListener('selectionchange', throttle(this.handleSelection, THROTTLE_DELAY));
        document.removeEventListener('mousedown', this.handleMouseDown);
        document.removeEventListener('copy', this.handleCopy);
    };
    /**
     * Adding progress listener for ocr.
     *
     * @param {ProgressListener} listener Callback function.
     */
    Ocr.prototype.addProgressListener = function (listener) {
        this.progressListeners.push(listener);
    };
    /**
     * Removing progress listener for ocr.
     *
     * @param {ProgressListener} listener Callback function that was added before.
     */
    Ocr.prototype.removeProgressListener = function (listener) {
        this.progressListeners = this.progressListeners.filter(function (item) { return item !== listener; });
    };
    /**
     * Parse image with ocr.
     *
     * @param {ImageLike} image The image instance.
     */
    Ocr.prototype.parseImage = function (image) {
        return __awaiter(this, void 0, void 0, function () {
            var worker, data;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        worker = createWorker({
                            logger: function (m) {
                                _this.progressListeners.forEach(function (callback) {
                                    callback(m);
                                });
                            },
                        });
                        return [4 /*yield*/, worker.load()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, worker.loadLanguage(this.language)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, worker.initialize(this.language)];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, worker.setParameters({
                                tessedit_pageseg_mode: "2" /* PSM.AUTO_ONLY */,
                            })];
                    case 4:
                        _a.sent();
                        return [4 /*yield*/, worker.recognize(image)];
                    case 5:
                        data = (_a.sent()).data;
                        this.data = data;
                        return [4 /*yield*/, worker.terminate()];
                    case 6:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    Ocr.prototype.updateTextSize = function (el) {
        var wrap = document.createElement('span');
        document.body.appendChild(wrap);
        wrap.innerHTML = el.textContent.trim();
        var height = parseInt(el.style.height.replace('px', ''));
        var width = parseInt(el.style.width.replace('px', ''));
        var fontSize = height;
        if (fontSize > this.minFontSize) {
            wrap.style.fontSize = "".concat(fontSize, "px");
            var fontSizeH = fontSize * height / wrap.offsetHeight;
            var fontSizeW = fontSize * width / wrap.offsetWidth;
            fontSize = fontSizeH < fontSizeW ? fontSizeH : fontSizeW;
        }
        if (fontSize < this.minFontSize) {
            fontSize = this.minFontSize;
        }
        el.style.fontSize = "".concat(fontSize, "px");
        wrap.remove();
    };
    Ocr.prototype.updateElementSize = function (el, _a, scale) {
        var x0 = _a.x0, x1 = _a.x1, y0 = _a.y0, y1 = _a.y1;
        el.style.width = "".concat((x1 - x0) * scale, "px");
        el.style.height = "".concat((y1 - y0) * scale, "px");
    };
    Ocr.prototype.mapWords = function (words, scale, parentBbox) {
        var wordElementsList = [];
        for (var i = 0; i < words.length; i++) {
            var _a = words[i], bbox = _a.bbox, text = _a.text;
            var nextBboxX = i < words.length - 1 ? words[i + 1].bbox.x0 : bbox.x1;
            var elementBbox = { x0: bbox.x0, x1: nextBboxX, y0: parentBbox.y0, y1: parentBbox.y1 };
            if (!text.trim().length)
                continue;
            var el = document.createElement('span');
            this.updateElementSize(el, elementBbox, scale);
            el.innerText = text;
            el.classList.add(WORD_CLASS_NAME);
            if (this.wordClassName.length) {
                el.classList.add(this.wordClassName);
            }
            this.updateTextSize(el);
            wordElementsList.push(el);
        }
        return wordElementsList;
    };
    Ocr.prototype.mapLine = function (line, scale) {
        var bbox = line.bbox, words = line.words;
        var el = document.createElement('span');
        this.updateElementSize(el, bbox, scale);
        el.style.display = 'flex';
        if (this.lineClassName.length) {
            el.classList.add(this.lineClassName);
        }
        var wordElements = this.mapWords(words, scale, bbox);
        if (!wordElements.length)
            return null;
        wordElements.forEach(function (word) {
            el.appendChild(word);
        });
        return { element: el, bbox: bbox };
    };
    Ocr.prototype.mapParagraph = function (paragraph, scale) {
        var bbox = paragraph.bbox, lines = paragraph.lines;
        var el = document.createElement('p');
        this.updateElementSize(el, bbox, scale);
        el.style.position = 'absolute';
        el.style.left = "".concat(bbox.x0 * scale, "px");
        el.style.top = "".concat(bbox.y0 * scale, "px");
        el.style.margin = '0px';
        var lineElements = [];
        for (var i = 0; i < lines.length; i++) {
            var line = this.mapLine(lines[i], scale);
            if (line) {
                lineElements.push(line);
            }
        }
        if (!lineElements.length)
            return null;
        for (var i = 0; i < lineElements.length; i++) {
            var _a = lineElements[i], element = _a.element, bbox_1 = _a.bbox;
            if (i > 0) {
                var marginTop = bbox_1.y0 - lineElements[i - 1].bbox.y1;
                element.style.marginTop = "".concat(marginTop, "px");
            }
            el.appendChild(element);
        }
        return { element: el, bbox: bbox };
    };
    /**
     * Append generated html to the container.
     *
     * @param {HTMLElement} container The container where elements should append.
     * @param {number} scale Scale of the container compared to original image size.
     */
    Ocr.prototype.appendHtml = function (container, scale) {
        if (scale === void 0) { scale = 1; }
        if (!this.data.hocr)
            return;
        var elements = [];
        var paragraphs = this.data.paragraphs;
        for (var i = 0; i < paragraphs.length; i++) {
            var paragraph = this.mapParagraph(paragraphs[i], scale);
            if (paragraph) {
                elements.push(paragraph);
            }
        }
        for (var i = 0; i < elements.length; i++) {
            var _a = elements[i], element = _a.element, bbox = _a.bbox;
            if (i < elements.length - 1) {
                var padding = elements[i + 1].bbox.y0 - bbox.y1;
                if (padding > 0) {
                    element.style.paddingBottom = "".concat(padding, "px");
                }
            }
            container.appendChild(element);
        }
    };
    Ocr.prototype.handleSelection = function () {
        var _a;
        var selection = document.getSelection();
        if ((_a = selection === null || selection === void 0 ? void 0 : selection.focusNode) === null || _a === void 0 ? void 0 : _a.parentNode) {
            var selectableElements = document.querySelectorAll(".".concat(WORD_CLASS_NAME));
            for (var _i = 0, _b = Array.from(selectableElements); _i < _b.length; _i++) {
                var element = _b[_i];
                if (!!selection.toString() && selection.containsNode(element, true)) {
                    element.style.background = '#ACCEF7';
                }
                else {
                    element.style.background = 'none';
                }
            }
        }
    };
    Ocr.prototype.handleMouseDown = function () {
        var selection = document.getSelection();
        if (!selection)
            return;
        var selectableElements = document.querySelectorAll(".".concat(WORD_CLASS_NAME));
        for (var _i = 0, _a = Array.from(selectableElements); _i < _a.length; _i++) {
            var element = _a[_i];
            if (selection.containsNode(element, true)) {
                element.style.background = 'none';
            }
        }
    };
    Ocr.prototype.handleCopy = function (event) {
        var selection = document.getSelection();
        if (!selection)
            return;
        var selectableElements = document.querySelectorAll(".".concat(WORD_CLASS_NAME));
        var output = '';
        for (var _i = 0, _a = Array.from(selectableElements); _i < _a.length; _i++) {
            var element = _a[_i];
            if (selection.containsNode(element, true)) {
                output += "".concat(element.textContent, " ");
                var blockChildNodes = element.closest('p').childNodes;
                var lineChildNodes = blockChildNodes[blockChildNodes.length - 1].childNodes;
                var lastLineChildNode = lineChildNodes[lineChildNodes.length - 1];
                if (lastLineChildNode.isEqualNode(element)) {
                    output += "\n";
                }
            }
        }
        event.clipboardData.setData('text/plain', output);
        event.preventDefault();
    };
    return Ocr;
}());
export default Ocr;
